const mongoose = require("mongoose");
const BitlyClient = require("bitly").BitlyClient;

const Message = mongoose.model("Message");
const Verify = mongoose.model("Verify");
const Merchant = mongoose.model("Merchant");

require("dotenv").config();

const {
  MESSAGE_PENDING_STATUS,
  MESSAGE_FAILED_STATUS,
  MESSAGE_DELIVERED_STATUS,
} = require("./status.helper");

const { TWILIO_API_KEY, TWILIO_AUTH_TOKEN, BITLY_ACCSESS_TOKEN } = process.env;

const twilio = require("twilio")(TWILIO_API_KEY, TWILIO_AUTH_TOKEN);
const bitlyToken = BITLY_ACCSESS_TOKEN || '32da1bd1e1302fdeb56e3197dfaed4e8b32034d1'
const bitly = new BitlyClient(bitlyToken);
/**
 *
 * @param {Merchant} merchant
 * @param {string} phoneNumber
 */
const sendVerificationCodeMessage = async (merchant, phoneNumber) => {
  const verification = new Verify();
  await verification.save();

  const message = new Message({
    merchant: merchant._id,
    from: merchant.longcode,
    to: phoneNumber,
    body: `Your TextAnOffer verification code is: ${verification.code}`,
    status: MESSAGE_PENDING_STATUS,
    type: "verification",
  });

  await message.save();

  await dispatchMessage(message);
};

/**
 *
 * @param {Merchant} merchant
 * @param {string} phoneNumber
 * @param {string} body
 */
const sendCampaignPreviewMessage = async (merchant, phoneNumber, body) => {
  const message = new Message({
    merchant: merchant._id,
    from: merchant.longcode,
    to: phoneNumber,
    body: `[PREVIEW] ${body}`,
    status: MESSAGE_PENDING_STATUS,
    type: "preview",
  });
console.log(" preview ",message)
  await message.save();

  await dispatchMessage(message);
};
/**
 *
 * @param {Subscriber} subscriber
 * @param {Merchant} merchant
 */
const sendNewSubscriberMessage = async (subscriber, merchant) => {
  const body = `New Text An Offer subscriber: ${subscriber.firstName} ${subscriber.lastName} ${subscriber.nationalFormat}`;
  const message = new Message({
    merchant: merchant._id,
    from: merchant.longcode,
    to: merchant.phoneNumber,
    body,
    status: MESSAGE_PENDING_STATUS,
    type: "new-subscriber",
  });

  await message.save();
  await dispatchMessage(message);
};
/**
 *
 * @param {Subscriber} subscriber
 */
const sendWelcomeMessage = async (subscriber, bycsv) => {
  const merchant = await Merchant.findById(subscriber.merchant);

  const sid = subscriber._id;
  var body;

  if (bycsv) {
    const url = `https://${merchant.subdomain}.textanoffer.com/?sid=${sid}`;
    const sortenUrl = await generateSortenUrl(url);

    body = `${merchant.name}:\n\nThanks for signing up, ${subscriber.firstName}. We look forward to sending you the best offers in the world!\n\n${merchant.name} sends occasional messages. Msg & data rates may apply. Text HELP for Help\n\nText STOP to opt-out\n\n please click on the link to fulfill the requirement ${sortenUrl.link}`;
  } else {
    body = `${merchant.name}:\n\nThanks for signing up, ${subscriber.firstName}. We look forward to sending you the best offers in the world!\n\n${merchant.name} sends occasional messages. Msg & data rates may apply. Text HELP for Help\n\nText stop to opt-out`;
  }
    const message = new Message({
    merchant: merchant._id,
    from: merchant.longcode,
    to: subscriber.phoneNumber,
    body,
    status: MESSAGE_PENDING_STATUS,
    subscriber: subscriber._id,
    subscriberInfo: {
      firstName: subscriber.firstName,
      lastName: subscriber.lastName,
    },
    nationalFormat: subscriber.nationalFormat,
    type: "welcome",
  });

  subscriber.messages += 1;

  await message.save();
  await subscriber.save();
  await dispatchMessage(message);

};

/**
 *
 * @param {Subscriber} subscriber
 * @param {Campaign} campaign
 * @param {Merchant} merchant
 */
const sendCampaignMessage = async (subscriber, campaign, merchant) => {
  console.log("in side sendCampaignMessage", campaign);
  var campaignType;
  if (campaign.price === 0 || campaign.price <= 0) {
    campaignType = "textCampaign";
  } else {
    campaignType = "campaign";
  }
  const message = new Message({
    merchant: merchant._id,
    campaign: campaign._id,
    from: merchant.longcode,
    to: subscriber.phoneNumber,
    media: campaign.media ? `${process.env.CDN_URL}/${campaign.media}` : "",
    body: campaign.message,
    status: MESSAGE_PENDING_STATUS,
    subscriber: subscriber._id,
    subscriberInfo: {
      firstName: subscriber.firstName,
      lastName: subscriber.lastName,
    },
    nationalFormat: subscriber.nationalFormat,
    type: campaignType,
  });

  subscriber.messages += 1;

  await message.save();
  console.log("message save");

  await subscriber.save();
  console.log("subscriber save");

  console.log("calling function for send message");
  await dispatchMessage(message);
};

/**
 *
 * @param {Message} message
 */
const dispatchMessage = async (message) => {
  console.log("message data", message);
  const { to, from, body } = message;
  const messageIsFraudulant = await checkMessageForFraud(message);

  const messageData = { from, to, body };
  if (message.media) messageData.mediaUrl = [message.media];
  if (message.campaign)
    messageData.statusUrl = `${process.env.API_URL}/message/incoming/${message.subscriber}/${message.campaign}`;

  if (messageIsFraudulant) {
    /**
     * Handling should consist of further checks on the account
     * and / or disabling the account until manual review can be
     * conducted.
     *
     *
     */
  }

  try {
    const twilioRequest = await twilio.messages.create(messageData);
    console.log("twilio message creation", twilioRequest);
    message.status = MESSAGE_DELIVERED_STATUS;
    message.twilio.id = twilioRequest.sid;

    await message.save();
  } catch (e) {
    console.log("twilio message creation error", e.message);
    message.status = MESSAGE_FAILED_STATUS;
    await message.save();
    throw e;
  }
};

/**
 *
 * @param {Message} message
 */
const checkMessageForFraud = async (message) => false;

const generateSortenUrl = async (url) => {
  try {
    const response = await bitly.shorten(url);
    return response;
  } catch (error) {
    console.log("Error => ", error.message);
  }
};

module.exports = {
  sendVerificationCodeMessage,
  sendCampaignPreviewMessage,
  sendWelcomeMessage,
  sendCampaignMessage,
  sendNewSubscriberMessage,
};
