import { Schema } from '../schema';
/**
 * Defines how discounts are automatically applied to a set of items that match the pricing rule
 * during the active time period.
 */
export interface CatalogPricingRule {
    /**
     * User-defined name for the pricing rule. For example, "Buy one get one
     * free" or "10% off".
     */
    name?: string;
    /**
     * A list of unique IDs for the catalog time periods when
     * this pricing rule is in effect. If left unset, the pricing rule is always
     * in effect.
     */
    timePeriodIds?: string[];
    /**
     * Unique ID for the `CatalogDiscount` to take off
     * the price of all matched items.
     */
    discountId?: string;
    /**
     * Unique ID for the `CatalogProductSet` that will be matched by this rule. A match rule
     * matches within the entire cart, and can match multiple times. This field will always be set.
     */
    matchProductsId?: string;
    /**
     * __Deprecated__: Please use the `exclude_products_id` field to apply
     * an exclude set instead. Exclude sets allow better control over quantity
     * ranges and offer more flexibility for which matched items receive a discount.
     * `CatalogProductSet` to apply the pricing to.
     * An apply rule matches within the subset of the cart that fits the match rules (the match set).
     * An apply rule can only match once in the match set.
     * If not supplied, the pricing will be applied to all products in the match set.
     * Other products retain their base price, or a price generated by other rules.
     */
    applyProductsId?: string;
    /**
     * `CatalogProductSet` to exclude from the pricing rule.
     * An exclude rule matches within the subset of the cart that fits the match rules (the match set).
     * An exclude rule can only match once in the match set.
     * If not supplied, the pricing will be applied to all products in the match set.
     * Other products retain their base price, or a price generated by other rules.
     */
    excludeProductsId?: string;
    /** Represents the date the Pricing Rule is valid from. Represented in RFC 3339 full-date format (YYYY-MM-DD). */
    validFromDate?: string;
    /**
     * Represents the local time the pricing rule should be valid from. Represented in RFC 3339 partial-time format
     * (HH:MM:SS). Partial seconds will be truncated.
     */
    validFromLocalTime?: string;
    /** Represents the date the Pricing Rule is valid until. Represented in RFC 3339 full-date format (YYYY-MM-DD). */
    validUntilDate?: string;
    /**
     * Represents the local time the pricing rule should be valid until. Represented in RFC 3339 partial-time format
     * (HH:MM:SS). Partial seconds will be truncated.
     */
    validUntilLocalTime?: string;
    /**
     * Indicates which products matched by a CatalogPricingRule
     * will be excluded if the pricing rule uses an exclude set.
     */
    excludeStrategy?: string;
}
export declare const catalogPricingRuleSchema: Schema<CatalogPricingRule>;
